@@ -1,89 +1,87 @@
-from __future__ import unicode_literals
-from future.utils import native_str
+import warnings
+from functools import lru_cache
 
 from django.contrib import admin
 from django.contrib.auth import logout
 from django.contrib.messages import error
 from django.contrib.redirects.models import Redirect
 from django.core.exceptions import MiddlewareNotUsed
-from django.core.urlresolvers import reverse, resolve
-from django.http import (HttpResponse, HttpResponseRedirect,
-                         HttpResponsePermanentRedirect, HttpResponseGone)
+from django.http import (
+    HttpResponse,
+    HttpResponseGone,
+    HttpResponsePermanentRedirect,
+    HttpResponseRedirect,
+)
 from django.middleware.csrf import CsrfViewMiddleware, get_token
-from django.template import Template, RequestContext
+from django.template import RequestContext, Template
+from django.urls import resolve, reverse
 from django.utils.cache import get_max_age
 from django.utils.safestring import mark_safe
-from django.utils.translation import ugettext as _
+from django.utils.translation import gettext as _
 
 from mezzanine.conf import settings
+from mezzanine.core.management.commands.createdb import (
+    DEFAULT_PASSWORD,
+    DEFAULT_USERNAME,
+)
 from mezzanine.core.models import SitePermission
-from mezzanine.core.management import DEFAULT_USERNAME, DEFAULT_PASSWORD
-from mezzanine.utils.cache import (cache_key_prefix, nevercache_token,
-                                   cache_get, cache_set, cache_installed)
-from mezzanine.utils.device import templates_for_device
-from mezzanine.utils.sites import current_site_id, templates_for_host
+from mezzanine.utils.cache import (
+    cache_get,
+    cache_installed,
+    cache_key_prefix,
+    cache_set,
+    nevercache_token,
+)
+from mezzanine.utils.conf import middlewares_or_subclasses_installed
+from mezzanine.utils.deprecation import MiddlewareMixin, is_authenticated
+from mezzanine.utils.sites import current_site_id
 from mezzanine.utils.urls import next_url
 
 
-_deprecated = {
-    "AdminLoginInterfaceSelector": "AdminLoginInterfaceSelectorMiddleware",
-    "DeviceAwareUpdateCacheMiddleware": "UpdateCacheMiddleware",
-    "DeviceAwareFetchFromCacheMiddleware": "FetchFromCacheMiddleware",
-}
-
-
-class _Deprecated(object):
-    def __init__(self, *args, **kwargs):
-        from warnings import warn
-        msg = "mezzanine.core.middleware.%s is deprecated." % self.old
-        if self.new:
-            msg += (" Please change the MIDDLEWARE_CLASSES setting to use "
-                    "mezzanine.core.middleware.%s" % self.new)
-        warn(msg)
-
-for old, new in _deprecated.items():
-    globals()[old] = type(native_str(old),
-                          (_Deprecated,),
-                          {"old": old, "new": new})
-
-
-class AdminLoginInterfaceSelectorMiddleware(object):
+class AdminLoginInterfaceSelectorMiddleware(MiddlewareMixin):
     """
     Checks for a POST from the admin login view and if authentication is
     successful and the "site" interface is selected, redirect to the site.
     """
+
     def process_view(self, request, view_func, view_args, view_kwargs):
         login_type = request.POST.get("mezzanine_login_interface")
-        if login_type and not request.user.is_authenticated():
+        if login_type and not is_authenticated(request.user):
             response = view_func(request, *view_args, **view_kwargs)
-            if request.user.is_authenticated():
+            if is_authenticated(request.user):
                 if login_type == "admin":
-                    next = request.get_full_path()
-                    try:
-                        username = request.user.get_username()
-                    except AttributeError:  # Django < 1.5
-                        username = request.user.username
-                    if (username == DEFAULT_USERNAME and
-                            request.user.check_password(DEFAULT_PASSWORD)):
-                        error(request, mark_safe(_(
-                              "Your account is using the default password, "
-                              "please <a href='%s'>change it</a> immediately.")
-                              % reverse("user_change_password",
-                                        args=(request.user.id,))))
+                    next = next_url(request) or request.get_full_path()
+                    username = request.user.get_username()
+                    if username == DEFAULT_USERNAME and request.user.check_password(
+                        DEFAULT_PASSWORD
+                    ):
+                        error(
+                            request,
+                            mark_safe(
+                                _(
+                                    "Your account is using the default password, "
+                                    "please <a href='%s'>change it</a> immediately."
+                                )
+                                % reverse(
+                                    "user_change_password", args=(request.user.id,)
+                                )
+                            ),
+                        )
                 else:
-                    next = next_url(request) or "/"
+                    next = "/"
                 return HttpResponseRedirect(next)
             else:
                 return response
         return None
 
 
-class SitePermissionMiddleware(object):
+class SitePermissionMiddleware(MiddlewareMixin):
     """
     Marks the current user with a ``has_site_permission`` which is
     used in place of ``user.is_staff`` to achieve per-site staff
     access.
     """
+
     def process_view(self, request, view_func, view_args, view_kwargs):
         has_site_permission = False
         if request.user.is_superuser:
@@ -104,33 +102,38 @@ class SitePermissionMiddleware(object):
         request.user.has_site_permission = has_site_permission
 
 
-class TemplateForDeviceMiddleware(object):
+class TemplateForDeviceMiddleware(MiddlewareMixin):
     """
+    DEPRECATED: Device detection has been removed from Mezzanine.
     Inserts device-specific templates to the template list.
     """
-    def process_template_response(self, request, response):
-        if hasattr(response, "template_name"):
-            if not isinstance(response.template_name, Template):
-                templates = templates_for_device(request,
-                    response.template_name)
-                response.template_name = templates
-        return response
+
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        warnings.warn(
+            "`TemplateForDeviceMiddleware` is deprecated. "
+            "Please remove it from your middleware settings.",
+            FutureWarning,
+            stacklevel=2,
+        )
 
 
-class TemplateForHostMiddleware(object):
+class TemplateForHostMiddleware(MiddlewareMixin):
     """
     Inserts host-specific templates to the template list.
     """
-    def process_template_response(self, request, response):
-        if hasattr(response, "template_name"):
-            if not isinstance(response.template_name, Template):
-                templates = templates_for_host(request,
-                    response.template_name)
-                response.template_name = templates
-        return response
+
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        warnings.warn(
+            "`TemplateForHostMiddleware` is deprecated. Please upgrade "
+            "to the template loader. See: https://goo.gl/SzHPR4",
+            FutureWarning,
+            stacklevel=2,
+        )
 
 
-class UpdateCacheMiddleware(object):
+class UpdateCacheMiddleware(MiddlewareMixin):
     """
     Response phase for Mezzanine's cache middleware. Handles caching
     the response, and then performing the second phase of rendering,
@@ -155,7 +158,7 @@ class UpdateCacheMiddleware(object):
         # and the response mustn't include an expiry age, indicating it
         # shouldn't be cached.
         marked_for_update = getattr(request, "_update_cache", False)
-        anon = hasattr(request, "user") and not request.user.is_authenticated()
+        anon = hasattr(request, "user") and not is_authenticated(request.user)
         timeout = get_max_age(response)
         if timeout is None:
             timeout = settings.CACHE_MIDDLEWARE_SECONDS
@@ -173,7 +176,7 @@ class UpdateCacheMiddleware(object):
         # enclosed by it, to avoid possible template code injection.
         token = nevercache_token()
         try:
-            token = token.encode('utf-8')
+            token = token.encode("utf-8")
         except AttributeError:
             pass
         parts = response.content.split(token)
@@ -192,17 +195,30 @@ class UpdateCacheMiddleware(object):
         context = RequestContext(request)
         for i, part in enumerate(parts):
             if i % 2:
-                part = Template(part).render(context).encode("utf-8")
+                part = Template(part.decode("utf-8")).render(context).encode("utf-8")
             parts[i] = part
         response.content = b"".join(parts)
         response["Content-Length"] = len(response.content)
-        if hasattr(request, '_messages'):
+        if hasattr(request, "_messages"):
             # Required to clear out user messages.
             request._messages.update(response)
+        # Response needs to be run-through the CSRF middleware again so
+        # that if there was a {% csrf_token %} inside of the nevercache
+        # the cookie will be correctly set for the the response
+        if csrf_middleware_installed():
+            response.csrf_processing_done = False
+            csrf_mw = CsrfViewMiddleware()
+            csrf_mw.process_response(request, response)
         return response
 
 
-class FetchFromCacheMiddleware(object):
+@lru_cache(maxsize=None)
+def csrf_middleware_installed():
+    csrf_mw_name = "django.middleware.csrf.CsrfViewMiddleware"
+    return middlewares_or_subclasses_installed([csrf_mw_name])
+
+
+class FetchFromCacheMiddleware(MiddlewareMixin):
     """
     Request phase for Mezzanine cache middleware. Return a response
     from cache if found, othwerwise mark the request for updating
@@ -210,15 +226,17 @@ class FetchFromCacheMiddleware(object):
     """
 
     def process_request(self, request):
-        if (cache_installed() and request.method == "GET" and
-            not request.user.is_authenticated()):
+        if (
+            cache_installed()
+            and request.method == "GET"
+            and not is_authenticated(request.user)
+        ):
             cache_key = cache_key_prefix(request) + request.get_full_path()
             response = cache_get(cache_key)
             # We need to force a csrf token here, as new sessions
             # won't receieve one on their first request, with cache
             # middleware running.
-            csrf_mw_name = "django.middleware.csrf.CsrfViewMiddleware"
-            if csrf_mw_name in settings.MIDDLEWARE_CLASSES:
+            if csrf_middleware_installed():
                 csrf_mw = CsrfViewMiddleware()
                 csrf_mw.process_view(request, lambda x: None, None, None)
                 get_token(request)
@@ -228,7 +246,7 @@ class FetchFromCacheMiddleware(object):
                 return HttpResponse(response)
 
 
-class SSLRedirectMiddleware(object):
+class SSLRedirectMiddleware(MiddlewareMixin):
     """
     Handles redirections required for SSL when ``SSL_ENABLED`` is ``True``.
 
@@ -238,16 +256,33 @@ class SSLRedirectMiddleware(object):
     Also ensure URLs defined by ``SSL_FORCE_URL_PREFIXES`` are redirect
     to HTTPS, and redirect all other URLs to HTTP if on HTTPS.
     """
+
+    def __init__(self, *args):
+        warnings.warn(
+            "SSLRedirectMiddleware is deprecated. See "
+            "https://docs.djangoproject.com/en/stable/ref/middleware/"
+            "#module-django.middleware.security for alternative solutions.",
+            DeprecationWarning,
+        )
+        super().__init__(*args)
+
+    def languages(self):
+        if not hasattr(self, "_languages"):
+            self._languages = dict(settings.LANGUAGES).keys()
+        return self._languages
+
     def process_request(self, request):
-        settings.use_editable()
         force_host = settings.SSL_FORCE_HOST
         response = None
         if force_host and request.get_host().split(":")[0] != force_host:
-            url = "http://%s%s" % (force_host, request.get_full_path())
+            url = f"http://{force_host}{request.get_full_path()}"
             response = HttpResponsePermanentRedirect(url)
         elif settings.SSL_ENABLED and not settings.DEV_SERVER:
-            url = "%s%s" % (request.get_host(), request.get_full_path())
-            if request.path.startswith(settings.SSL_FORCE_URL_PREFIXES):
+            url = f"{request.get_host()}{request.get_full_path()}"
+            path = request.path
+            if settings.USE_I18N and path[1:3] in self.languages():
+                path = path[3:]
+            if path.startswith(settings.SSL_FORCE_URL_PREFIXES):
                 if not request.is_secure():
                     response = HttpResponseRedirect("https://%s" % url)
             elif request.is_secure() and settings.SSL_FORCED_PREFIXES_ONLY:
@@ -268,13 +303,14 @@ class SSLRedirectMiddleware(object):
         return response
 
 
-class RedirectFallbackMiddleware(object):
+class RedirectFallbackMiddleware(MiddlewareMixin):
     """
     Port of Django's ``RedirectFallbackMiddleware`` that uses
     Mezzanine's approach for determining the current site.
     """
 
-    def __init__(self):
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
         if "django.contrib.redirects" not in settings.INSTALLED_APPS:
             raise MiddlewareNotUsed
 
@@ -292,5 +328,5 @@ class RedirectFallbackMiddleware(object):
                 if not redirect.new_path:
                     response = HttpResponseGone()
                 else:
-                    response = HttpResponseRedirect(redirect.new_path)
+                    response = HttpResponsePermanentRedirect(redirect.new_path)
         return response
