@@ -1,7 +1,6 @@
-from __future__ import unicode_literals
-
 import threading
 
+from mezzanine.utils.deprecation import MiddlewareMixin
 
 _thread_local = threading.local()
 
@@ -13,7 +12,7 @@ def current_request():
     return getattr(_thread_local, "request", None)
 
 
-class CurrentRequestMiddleware(object):
+class CurrentRequestMiddleware(MiddlewareMixin):
     """
     Stores the request in the current thread for global access.
     """
