@@ -1,22 +1,23 @@
-from __future__ import unicode_literals
-from future.builtins import str
-
 from json import dumps
+from string import punctuation
 
+from django.apps import apps
 from django.contrib.admin.views.decorators import staff_member_required
 from django.contrib.messages import error
-from django.core.urlresolvers import reverse
 from django.db.models import ObjectDoesNotExist
-from django.http import HttpResponse
+from django.http import HttpResponse, HttpResponseBadRequest
 from django.shortcuts import redirect
-from django.utils.translation import ugettext_lazy as _
+from django.template.response import TemplateResponse
+from django.urls import reverse
+from django.utils.translation import gettext_lazy as _
 
 from mezzanine.conf import settings
-from mezzanine.generic.forms import ThreadedCommentForm, RatingForm
+from mezzanine.generic.forms import RatingForm, ThreadedCommentForm
 from mezzanine.generic.models import Keyword
 from mezzanine.utils.cache import add_cache_bypass
-from mezzanine.utils.models import get_model
-from mezzanine.utils.views import render, set_cookie, is_spam
+from mezzanine.utils.deprecation import is_authenticated
+from mezzanine.utils.importing import import_dotted_path
+from mezzanine.utils.views import is_spam, set_cookie
 
 
 @staff_member_required
@@ -27,15 +28,19 @@ def admin_keywords_submit(request):
     keywords field.
     """
     keyword_ids, titles = [], []
+    remove = punctuation.replace("-", "")  # Strip punctuation, allow dashes.
     for title in request.POST.get("text_keywords", "").split(","):
-        title = "".join([c for c in title if c.isalnum() or c in "- "]).strip()
+        title = "".join(c for c in title if c not in remove).strip()
         if title:
             kw, created = Keyword.objects.get_or_create_iexact(title=title)
             keyword_id = str(kw.id)
             if keyword_id not in keyword_ids:
                 keyword_ids.append(keyword_id)
                 titles.append(title)
-    return HttpResponse("%s|%s" % (",".join(keyword_ids), ", ".join(titles)))
+    return HttpResponse(
+        "{}|{}".format(",".join(keyword_ids), ", ".join(titles)),
+        content_type="text/plain",
+    )
 
 
 def initial_validation(request, prefix):
@@ -55,21 +60,29 @@ def initial_validation(request, prefix):
     ratings view functions to deal with as needed.
     """
     post_data = request.POST
-    settings.use_editable()
     login_required_setting_name = prefix.upper() + "S_ACCOUNT_REQUIRED"
     posted_session_key = "unauthenticated_" + prefix
     redirect_url = ""
     if getattr(settings, login_required_setting_name, False):
-        if not request.user.is_authenticated():
-            request.session[posted_session_key] = request.POST
-            error(request, _("You must be logged in. Please log in or "
-                             "sign up to complete this action."))
-            redirect_url = "%s?next=%s" % (settings.LOGIN_URL, reverse(prefix))
+        if not is_authenticated(request.user):
+            if request.method == "POST":
+                request.session[posted_session_key] = request.POST
+                error(
+                    request,
+                    _(
+                        "You must be logged in. Please log in or "
+                        "sign up to complete this action."
+                    ),
+                )
+            redirect_url = f"{settings.LOGIN_URL}?next={reverse(prefix)}"
         elif posted_session_key in request.session:
             post_data = request.session.pop(posted_session_key)
     if not redirect_url:
+        model_data = post_data.get("content_type", "").split(".", 1)
+        if len(model_data) != 2:
+            return HttpResponseBadRequest()
         try:
-            model = get_model(*post_data.get("content_type", "").split(".", 1))
+            model = apps.get_model(*model_data)
             obj = model.objects.get(id=post_data.get("object_pk", None))
         except (TypeError, ObjectDoesNotExist, LookupError):
             redirect_url = "/"
@@ -81,7 +94,7 @@ def initial_validation(request, prefix):
     return obj, post_data
 
 
-def comment(request, template="generic/comments.html"):
+def comment(request, template="generic/comments.html", extra_context=None):
     """
     Handle a ``ThreadedCommentForm`` submission and redirect back to its
     related object.
@@ -90,7 +103,8 @@ def comment(request, template="generic/comments.html"):
     if isinstance(response, HttpResponse):
         return response
     obj, post_data = response
-    form = ThreadedCommentForm(request, obj, post_data)
+    form_class = import_dotted_path(settings.COMMENT_FORM_CLASS)
+    form = form_class(request, obj, post_data)
     if form.is_valid():
         url = obj.get_absolute_url()
         if is_spam(request, form, url):
@@ -107,8 +121,8 @@ def comment(request, template="generic/comments.html"):
         return HttpResponse(dumps({"errors": form.errors}))
     # Show errors with stand-alone comment form.
     context = {"obj": obj, "posted_comment_form": form}
-    response = render(request, template, context)
-    return response
+    context.update(extra_context or {})
+    return TemplateResponse(request, template, context)
 
 
 def rating(request):
@@ -131,8 +145,11 @@ def rating(request):
             rating_name = obj.get_ratingfield_name()
             json = {}
             for f in ("average", "count", "sum"):
-                json["rating_" + f] = getattr(obj, "%s_%s" % (rating_name, f))
+                json["rating_" + f] = getattr(obj, f"{rating_name}_{f}")
             response = HttpResponse(dumps(json))
-        ratings = ",".join(rating_form.previous + [rating_form.current])
-        set_cookie(response, "mezzanine-rating", ratings)
+        if rating_form.undoing:
+            ratings = set(rating_form.previous) ^ {rating_form.current}
+        else:
+            ratings = rating_form.previous + [rating_form.current]
+        set_cookie(response, "mezzanine-rating", ",".join(ratings))
     return response
