@@ -1,29 +1,37 @@
-from __future__ import unicode_literals
-
 import os
 import sys
+from inspect import getmro
 from warnings import warn
 
-from django import VERSION
 from django.conf import global_settings as defaults
-from django.template.base import add_to_builtins
+from django.db import OperationalError
+from django.utils.module_loading import import_string
 
+from mezzanine.utils.deprecation import get_middleware_setting
 from mezzanine.utils.timezone import get_best_local_timezone
 
 
-class SitesAllowedHosts(object):
+class SitesAllowedHosts:
     """
-    This is a fallback for Django 1.5's ALLOWED_HOSTS setting
+    This is a fallback for Django's ALLOWED_HOSTS setting
     which is required when DEBUG is False. It looks up the
     ``Site`` model and uses any domains added to it, the
     first time the setting is accessed.
     """
+
     def __iter__(self):
         if getattr(self, "_hosts", None) is None:
             from django.contrib.sites.models import Site
-            self._hosts = [s.domain.split(":")[0] for s in Site.objects.all()]
+
+            try:
+                self._hosts = [s.domain.split(":")[0] for s in Site.objects.all()]
+            except OperationalError:
+                return iter([])
         return iter(self._hosts)
 
+    def __add__(self, other):
+        return list(self) + other
+
 
 def set_dynamic_settings(s):
     """
@@ -45,13 +53,12 @@ def set_dynamic_settings(s):
     # Remove a value from a list setting if in the list.
     remove = lambda n, k: s[n].remove(k) if k in s[n] else None
 
-    s["TEMPLATE_DEBUG"] = s.get("TEMPLATE_DEBUG", s.get("DEBUG", False))
-    add_to_builtins("mezzanine.template.loader_tags")
-
     if not s.get("ALLOWED_HOSTS", []):
-        warn("You haven't defined the ALLOWED_HOSTS settings, which "
-             "Django 1.5 requires. Will fall back to the domains "
-             "configured as sites.")
+        warn(
+            "You haven't defined the ALLOWED_HOSTS settings, which "
+            "Django requires. Will fall back to the domains "
+            "configured as sites."
+        )
         s["ALLOWED_HOSTS"] = SitesAllowedHosts()
 
     if s.get("TIME_ZONE", None) is None:
@@ -70,9 +77,13 @@ def set_dynamic_settings(s):
     # Change tuple settings to lists for easier manipulation.
     s.setdefault("AUTHENTICATION_BACKENDS", defaults.AUTHENTICATION_BACKENDS)
     s.setdefault("STATICFILES_FINDERS", defaults.STATICFILES_FINDERS)
-    tuple_list_settings = ["AUTHENTICATION_BACKENDS", "INSTALLED_APPS",
-                           "MIDDLEWARE_CLASSES", "STATICFILES_FINDERS",
-                           "LANGUAGES", "TEMPLATE_CONTEXT_PROCESSORS"]
+    tuple_list_settings = [
+        "AUTHENTICATION_BACKENDS",
+        "INSTALLED_APPS",
+        "MIDDLEWARE",
+        "STATICFILES_FINDERS",
+        "LANGUAGES",
+    ]
     for setting in tuple_list_settings[:]:
         if not isinstance(s.get(setting, []), list):
             s[setting] = list(s[setting])
@@ -81,69 +92,74 @@ def set_dynamic_settings(s):
             # the list of settings we'll revert back to tuples.
             tuple_list_settings.remove(setting)
 
-    # From Mezzanine 3.1.2 and onward we added the context processor
-    # for handling the page variable in templates - here we help
-    # upgrading by adding it if missing, with a warning. This helper
-    # can go away eventually.
-    cp = "mezzanine.pages.context_processors.page"
-    if ("mezzanine.pages" in s["INSTALLED_APPS"] and
-            cp not in s["TEMPLATE_CONTEXT_PROCESSORS"]):
-        warn("%s is required in the TEMPLATE_CONTEXT_PROCESSORS setting. "
-             "Adding it now, but you should update settings.py to "
-             "explicitly include it." % cp)
-        append("TEMPLATE_CONTEXT_PROCESSORS", cp)
-
     # Set up cookie messaging if none defined.
     storage = "django.contrib.messages.storage.cookie.CookieStorage"
     s.setdefault("MESSAGE_STORAGE", storage)
 
-    if s["TESTING"]:
-        # Following bits are work-arounds for some assumptions that
-        # Django 1.5's tests make.
-
-        # contrib.auth tests fail without its own auth backend installed.
-        append("AUTHENTICATION_BACKENDS",
-               "django.contrib.auth.backends.ModelBackend")
-
-        # Tests in contrib.redirects simply don't work with a
-        # catch-all urlpattern such as Mezzanine's pages app.
-        remove("INSTALLED_APPS", "django.contrib.redirects")
-        remove("MIDDLEWARE_CLASSES",
-            "django.contrib.redirects.middleware.RedirectFallbackMiddleware")
-
+    # If required, add django-modeltranslation for both tests and deployment
+    if not s.get("USE_MODELTRANSLATION", False) or s["TESTING"]:
+        s["USE_MODELTRANSLATION"] = False
+        remove("INSTALLED_APPS", "modeltranslation")
     else:
-        # Setup for optional apps.
-        optional = list(s.get("OPTIONAL_APPS", []))
-        s["USE_SOUTH"] = s.get("USE_SOUTH") and VERSION < (1, 7)
-        if s.get("USE_SOUTH"):
-            optional.append("south")
-        elif not s.get("USE_SOUTH", True) and "south" in s["INSTALLED_APPS"]:
-            s["INSTALLED_APPS"].remove("south")
-        for app in optional:
-            if app not in s["INSTALLED_APPS"]:
-                try:
-                    __import__(app)
-                except ImportError:
-                    pass
-                else:
-                    s["INSTALLED_APPS"].append(app)
+        try:
+            __import__("modeltranslation")
+        except ImportError:
+            # django-modeltranslation is not installed, remove setting so
+            # admin won't try to import it
+            s["USE_MODELTRANSLATION"] = False
+            remove("INSTALLED_APPS", "modeltranslation")
+            warn(
+                "USE_MODETRANSLATION setting is set to True but django-"
+                "modeltranslation is not installed. Disabling it."
+            )
+        else:
+            # Force i18n so we are assured that modeltranslation is active
+            s["USE_I18N"] = True
+            append("INSTALLED_APPS", "modeltranslation")
+
+    # Setup for optional apps.
+    optional = list(s.get("OPTIONAL_APPS", []))
+    for app in optional:
+        if app not in s["INSTALLED_APPS"]:
+            try:
+                __import__(app)
+            except ImportError:
+                pass
+            else:
+                s["INSTALLED_APPS"].append(app)
+
+    if s["TESTING"]:
+        # Triggers interactive superuser creation and some pyc/pyo tests
+        # fail with standard permissions.
+        remove("INSTALLED_APPS", "django_extensions")
+
     if "debug_toolbar" in s["INSTALLED_APPS"]:
+        # We need to configure debug_toolbar manually otherwise it
+        # breaks in conjunction with modeltranslation. See the
+        # "Explicit setup" section in debug_toolbar docs for more info.
+        s["DEBUG_TOOLBAR_PATCH_SETTINGS"] = False
         debug_mw = "debug_toolbar.middleware.DebugToolbarMiddleware"
-        append("MIDDLEWARE_CLASSES", debug_mw)
+        append("MIDDLEWARE", debug_mw)
+        s.setdefault("INTERNAL_IPS", ("127.0.0.1",))
 
     # If compressor installed, ensure it's configured and make
     # Mezzanine's settings available to its offline context,
     # since jQuery is configured via a setting.
     if "compressor" in s["INSTALLED_APPS"]:
         append("STATICFILES_FINDERS", "compressor.finders.CompressorFinder")
-        s.setdefault("COMPRESS_OFFLINE_CONTEXT", {
-            "MEDIA_URL": s.get("MEDIA_URL", ""),
-            "STATIC_URL": s.get("STATIC_URL", ""),
-        })
+        s.setdefault(
+            "COMPRESS_OFFLINE_CONTEXT",
+            {
+                "MEDIA_URL": s.get("MEDIA_URL", ""),
+                "STATIC_URL": s.get("STATIC_URL", ""),
+            },
+        )
 
         def mezzanine_settings():
             from mezzanine.conf import settings
+
             return settings
+
         s["COMPRESS_OFFLINE_CONTEXT"]["settings"] = mezzanine_settings
 
     # Ensure the Mezzanine auth backend is enabled if
@@ -156,15 +172,6 @@ def set_dynamic_settings(s):
     # Ensure Grappelli is after Mezzanine in app order so that
     # admin templates are loaded in the correct order.
     grappelli_name = s.get("PACKAGE_NAME_GRAPPELLI")
-    if s["TESTING"]:
-        # Optional apps aren't installed when testing, but we need
-        # grappelli to perform some admin tests for certain HTML.
-        try:
-            __import__(grappelli_name)
-        except ImportError:
-            pass
-        else:
-            append("INSTALLED_APPS", grappelli_name)
     try:
         move("INSTALLED_APPS", grappelli_name, len(s["INSTALLED_APPS"]))
     except ValueError:
@@ -175,24 +182,18 @@ def set_dynamic_settings(s):
     # Ensure admin is at the bottom of the app order so that admin
     # templates are loaded in the correct order, and that staticfiles
     # is also at the end so its runserver can be overridden.
-    apps = ["django.contrib.admin"]
-    if VERSION >= (1, 7):
-        apps += ["django.contrib.staticfiles"]
-    for app in apps:
+    for app in ["django.contrib.admin", "django.contrib.staticfiles"]:
         try:
             move("INSTALLED_APPS", app, len(s["INSTALLED_APPS"]))
         except ValueError:
             pass
 
-    # Ensure we have a test runner (removed in Django 1.6)
-    s.setdefault("TEST_RUNNER", "django.test.simple.DjangoTestSuiteRunner")
-
     # Add missing apps if existing apps depend on them.
     if "mezzanine.blog" in s["INSTALLED_APPS"]:
         append("INSTALLED_APPS", "mezzanine.generic")
     if "mezzanine.generic" in s["INSTALLED_APPS"]:
         s.setdefault("COMMENTS_APP", "mezzanine.generic")
-        append("INSTALLED_APPS", "django.contrib.comments")
+        append("INSTALLED_APPS", "django_comments")
 
     # Ensure mezzanine.boot is first.
     try:
@@ -202,17 +203,35 @@ def set_dynamic_settings(s):
 
     # Remove caching middleware if no backend defined.
     if not (s.get("CACHE_BACKEND") or s.get("CACHES")):
-        s["MIDDLEWARE_CLASSES"] = [mw for mw in s["MIDDLEWARE_CLASSES"] if not
-                                   (mw.endswith("UpdateCacheMiddleware") or
-                                    mw.endswith("FetchFromCacheMiddleware"))]
+        s["MIDDLEWARE"] = [
+            mw
+            for mw in s["MIDDLEWARE"]
+            if not (
+                mw.endswith("UpdateCacheMiddleware")
+                or mw.endswith("FetchFromCacheMiddleware")
+            )
+        ]
 
     # If only LANGUAGE_CODE has been defined, ensure the other required
     # settings for translations are configured.
-    if (s.get("LANGUAGE_CODE") and len(s.get("LANGUAGES", [])) == 1 and
-            s["LANGUAGE_CODE"] != s["LANGUAGES"][0][0]):
+    if (
+        s.get("LANGUAGE_CODE")
+        and len(s.get("LANGUAGES", [])) == 1
+        and s["LANGUAGE_CODE"] != s["LANGUAGES"][0][0]
+    ):
         s["USE_I18N"] = True
         s["LANGUAGES"] = [(s["LANGUAGE_CODE"], "")]
 
+    # Ensure required middleware is installed, otherwise admin
+    # becomes inaccessible.
+    if s["USE_I18N"] and not [
+        mw for mw in s["MIDDLEWARE"] if mw.endswith("LocaleMiddleware")
+    ]:
+        session = s["MIDDLEWARE"].index(
+            "django.contrib.sessions.middleware.SessionMiddleware"
+        )
+        s["MIDDLEWARE"].insert(session + 1, "django.middleware.locale.LocaleMiddleware")
+
     # Revert tuple settings back to tuples.
     for setting in tuple_list_settings:
         s[setting] = tuple(s[setting])
@@ -220,11 +239,45 @@ def set_dynamic_settings(s):
     # Some settings tweaks for different DB engines.
     for (key, db) in s["DATABASES"].items():
         shortname = db["ENGINE"].split(".")[-1]
-        if shortname == "sqlite3" and os.sep not in db["NAME"]:
+        if shortname == "sqlite3":
             # If the Sqlite DB name doesn't contain a path, assume
             # it's in the project directory and add the path to it.
-            db_path = os.path.join(s.get("PROJECT_ROOT", ""), db["NAME"])
-            s["DATABASES"][key]["NAME"] = db_path
+            if "NAME" in db and os.sep not in db["NAME"]:
+                db_path = os.path.join(s.get("PROJECT_ROOT", ""), db["NAME"])
+                db["NAME"] = db_path
         elif shortname == "mysql":
             # Required MySQL collation for tests.
-            s["DATABASES"][key]["TEST_COLLATION"] = "utf8_general_ci"
+            db.setdefault("TEST", {})["COLLATION"] = "utf8_general_ci"
+
+
+def real_project_name(project_name):
+    """
+    Used to let Mezzanine run from its project template directory, in which
+    case "{{ project_name }}" won't have been replaced by a real project name.
+    """
+    if project_name == "{{ project_name }}":
+        return "project_name"
+    return project_name
+
+
+def middlewares_or_subclasses_installed(needed_middlewares):
+    """
+    When passed an iterable of dotted strings, returns True if all
+    of the middlewares (or their subclasses) are installed.
+    """
+    middleware_setting = set(get_middleware_setting())
+
+    # Shortcut case, check by string
+    if all(m in middleware_setting for m in needed_middlewares):
+        return True
+
+    def flatten(seqs):
+        return (item for seq in seqs for item in seq)
+
+    middleware_items = map(import_string, middleware_setting)
+    middleware_classes = filter(lambda m: isinstance(m, type), middleware_items)
+    middleware_ancestors = set(flatten(map(getmro, middleware_classes)))
+
+    needed_middlewares = set(map(import_string, needed_middlewares))
+
+    return needed_middlewares.issubset(middleware_ancestors)
